// Copyright (c) 2010  Anna Bigatti, Christof Soeger
// This file is part of the CoCoALib suite of examples.
// You are free to use any part of this example in your own programs.

#include "CoCoA/library.H"
#include<fstream>
using std::ifstream;

using namespace CoCoA;
using namespace std;

//----------------------------------------------------------------------
const string ShortDescription =
  "Reads a normaliz input file and computes the Hilbert basis.\n"
  "It also shows how to set some libnormaliz flags";

const string LongDescription =
  "the input file can be specified in the source code\n"
  "                                                           \n";
//----------------------------------------------------------------------


void program()
{
//  CoCoA::GMPAllocator sentinel(); // use CoCoALib's custom GMP allocator
  GlobalManager CoCoAFoundations;
  cout << ShortDescription << endl;
  cout << boolalpha; // prints true/false for bool

#ifdef CoCoA_WITH_NORMALIZ
  
  //set the output stream used to give warnings
  //(at the moment also error messages, but they will be transfered into the thrown Exception)
  libnormaliz::setErrorOutput(cerr);   //default: std::cerr
  
  //set the output stream used to give verbose output
  //verbose output will only be printed if libnormaliz::verbose is set to true
  libnormaliz::setVerboseOutput(cout); //default: std::cout
  libnormaliz::verbose = true;                   //default: false

  // we use integer type long, so we want to check for errors:
  libnormaliz::test_arithmetic_overflow = true;  //default: false

  //  ifstream fin("ex-Normaliz2-2.in",ifstream::in);  // fits in long, 40s on old Mac
  //  ifstream fin("ex-Normaliz2-1.in",ifstream::in);  // needs big integers
  int nr, nc;
  cin >> nr;
  cin >> nc;
  vector<vector<ZZ> > l;
  ZZ tmp;

  for(int i=0; i<nr; i++) {
    vector<ZZ> v;
    for(int j=0; j<nc; j++) {
      cin >> tmp;
      v.push_back(tmp);
    }
    l.push_back(v);
	}

  cout << "l -> " << l << endl;

  int InputType;
  cin >> InputType;

  NormalizCone c(NewNormalizConeLong(l, InputType));
  long t0 = CpuTime();
  HilbertBasis(l, c);
  cout << "time: " << CpuTime()-t0 << endl;
  
  cout << "HilbertBasis -> " << l << endl;
#else
  cout << "Normaliz library is not linked." << endl;
#endif

}





//----------------------------------------------------------------------
// Use main() to handle any uncaught exceptions and warn the user about them.
int main()
{
  try
  {
    program();
    return 0;
  }
  catch (const CoCoA::ErrorInfo& err)
  {
    cerr << "***ERROR***  UNCAUGHT CoCoA error";
    ANNOUNCE(cerr, err);
  }
  catch (const std::exception& exc)
  {
    cerr << "***ERROR***  UNCAUGHT std::exception: " << exc.what() << endl;
  }
  catch(...)
  {
    cerr << "***ERROR***  UNCAUGHT UNKNOWN EXCEPTION" << endl;
  }
  return 1;
}

//----------------------------------------------------------------------
// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/examples/ex-Normaliz2.C,v 1.8 2011/02/17 17:12:57 bigatti Exp $
// $Log: ex-Normaliz2.C,v $
// Revision 1.8  2011/02/17 17:12:57  bigatti
// -- reading input from cin
//
// Revision 1.7  2010/12/17 16:07:54  abbott
// Ensured that all i/o in examples is on standard C++ streams
// (rather than GlobalInput(), etc).
//
// Revision 1.6  2010/10/22 13:31:01  bigatti
// -- added comments about normaliz flags (by C.Soeger)
//
// Revision 1.5  2010/10/12 11:22:54  bigatti
// -- TmpNormaliz.H simplified:
//    now NormalizCone is a smart pointer to NormalizConeBase
//    and the concrete classes are entirely in the .C file
// -- added Ht1Elements, SupportHyperplanes, Triangulation
// -- added some text in ex-Normaliz1 and 2
//
// Revision 1.4  2010/10/08 16:33:27  bigatti
// -- added #include <fstream>
//
// Revision 1.3  2010/10/08 13:53:51  bigatti
// -- cleaned up
//
// Revision 1.2  2010/10/08 10:39:32  bigatti
// -- extended interface for normaliz
//
// Revision 1.1  2010/10/08 06:57:09  bigatti
// * ex-Normaliz2-1.in (Module): first import
//
// * ex-Normaliz2-2.in (Module): first import
//
// * ex-Normaliz2.C (Module): first import
//
// Revision 1.1  2010/10/05 14:31:26  bigatti
// -- first import
//
// Revision 1.4  2008/10/07 12:12:54  abbott
// Removed useless commented out #include.
//
// Revision 1.3  2007/05/31 16:06:16  bigatti
// -- removed previous unwanted checked-in version
//
// Revision 1.1.1.1  2007/03/09 15:16:11  abbott
// Imported files
//
// Revision 1.9  2007/03/07 11:51:40  bigatti
// -- improved test alignment
//
// Revision 1.8  2007/03/03 14:15:45  bigatti
// -- "foundations" renamed into "GlobalManager"
//
// Revision 1.7  2007/03/02 17:46:40  bigatti
// -- unique RingZ and RingQ
// -- requires foundations.H ;  foundations blah;  (thik of a better name)
//
// Revision 1.6  2007/03/02 10:47:53  cocoa
// First stage of RingZ modifications -- tests do not compile currently, Anna will fix this.
//
// Revision 1.5  2007/03/01 13:52:59  bigatti
// -- minor: fixed typo
//
// Revision 1.4  2007/02/28 15:15:56  bigatti
// -- minor: removed quotes in description
//
// Revision 1.3  2007/02/12 16:27:43  bigatti
// -- added strings ShortDescription and LongDescription for indexing
//
// Revision 1.2  2007/02/10 18:44:03  cocoa
// Added "const" twice to each test and example.
// Eliminated dependency on io.H in several files.
// Improved BuildInfo, and added an example about how to use it.
// Some other minor cleaning.
//
// Revision 1.1.1.1  2006/05/30 11:39:36  cocoa
// Imported files
//
// Revision 1.1  2006/03/12 21:28:34  cocoa
// Major check in after many changes
//
