//   Copyright (c)  2008-2009  Anna Bigatti and Eduardo Saenz-de-Cabezon

//   This file is part of the source of CoCoALib, the CoCoA Library.

//   CoCoALib is free software: you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation, either version 3 of the License, or
//   (at your option) any later version.

//   CoCoALib is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with CoCoALib.  If not, see <http://www.gnu.org/licenses/>.

#include "CoCoA/TmpMonomialFunctions.H"

#include "CoCoA/PPMonoid.H"  // for MultidegreeMap
#include "CoCoA/TmpPPVector.H"
#include "CoCoA/ZZ.H"  // for position_t
#include "CoCoA/io.H"
#include "CoCoA/tmp.H"

using std::size_t;
#include <iostream>

// using std::list;
// #include <utility>
// using std::pair;
using std::vector;
// using std::endl;

/**********************************************************/

namespace CoCoA
{
  void support(std::vector<long>& sup, const PPMonoidElem& monomial)
  {
    std::vector<long> expv;
    exponents(expv,monomial); 
    sup.clear();
    const long size=expv.size();
    for (long i=0; i<size; ++i)
      if (expv[i]!=0)
        sup.push_back(i);
  }

  bool IsIrreducible(const PPVector& ideal)
  {
    long numgens=len(ideal);
    for (long cont=0; cont<numgens; ++cont)
      if (!IsIndetPosPower(PP(ideal[cont])))
        return false;
    return true;
  }

  bool IsPrime(const PPVector& ideal)
  {
    long numgens=len(ideal);
    long var;
    for (long cont=0; cont<numgens; ++cont)
      if (!IsIndet(var, PP(ideal[cont])))
        return false;
    return true;
  }

  bool IsPrimary(const PPVector& ideal)
  {
    long numgens=len(ideal);
    long var;
    ZZ expo;
    long N=NumIndets(PPM(ideal));
    vector<bool> pures(N,false);
    vector<bool> other(N,false);
    for(long cont=0; cont<numgens; ++cont)
    {
      if (IsIndetPosPower(var, expo, PP(ideal[cont])))
        pures[var] = true;
      else //I copied part of "support"
      {
        std::vector<long> expv;
        exponents(expv, PP(ideal[cont]));
        for (long i=0; i<N; ++i)
          if (expv[i]!=0)
            other[i]=true;
      }
    }
    for (long conta=0; conta<N; ++conta)
      if (pures[conta]==false and other[conta]==true)
        return false;
    return true;
  }

  void ColonIdeal(PPVector& PPs, const PPVector& PPs1, const PPVector& PPs2)
  {
    //is it relevant which one has more generators?
    PushBack(PPs,one(PPM(PPs)));
    for(long i=0; i<len(PPs2); ++i)
    {
      PPVector current(PPM(PPs),DMR(PPs));
      for (long j=0; j<len(PPs1); ++j)
        PushBack(current, colon(PP(PPs1[j]),PP(PPs2[i])));
      interreduce(current); //is it better to just add and then interreduce at the end?
      lcms(PPs, PPs, current);
    }
    interreduce(PPs);
  }
  


}  // end of namespace CoCoA

// RCS header/log in the next few lines
// $Header: /Volumes/Home_1/cocoa/cvs-repository/CoCoALib-0.99/src/AlgebraicCore/TmpMonomialFunctions.C,v 1.7 2011/03/22 15:30:07 bigatti Exp $
// $Log: TmpMonomialFunctions.C,v $
// Revision 1.7  2011/03/22 15:30:07  bigatti
// -- added cvs log
//
